     ; Include derivative-specific definitions
     INCLUDE 'derivative.inc'

     ABSENTRY   Start

PAGEWINDOW  equ  $00C0

;********************************************************************************
; Fast RAM ($0 - $D)
                  org   $0
;            
; Programming parameters (values given are dummies for testing)
;
Destination    equ   $3F11
             
SourceAddr        dc.b  $08                             ; Address in Zero Page (Buffer) for source
Buffer            dc.b  $88,$77,$66,$55,$44,$33,$22,$11 ; Buffer for data to program
DestinationAddr   dc.b  MAP_ADDR_6(Destination)         ; Address in Zero Page (Paging Window) for destination
DestinationPage   dc.b  HIGH_6_13(Destination)          ; Page for destination
;
; Timing constants for Flash Programming 
;          
; Assumes bus freq = ~4 MHz clock => 250 ns, 4cy = 1us
;
Time7us          dc.b  4   ; PGM/MASS to HVEN setup time Tnvs >  5us,   12+4*N, 28cy = ~7us
                           ; HVEN hold time for PGM      Tnvh >  5us,    8+4*N, 24cy = ~6us
Time12us         dc.b  6   ; HVEN to program setup time  Tpgs > 10 us,  24+4*N, 48cy = ~12us 
Time30us         dc.b  23  ; Byte program time           Tprog(20-40us) 29+4*N, 81cy = ~30.25us
;                                                        Tprog(20-40us) 23+4*N, 75cy = ~28.75us
  
;********************************************************************************
; Flash programming code in Z-Page RAM
;            
   org   RAMStart

Start:
   mov   #HIGH_6_13(FLCR),PAGESEL   ; 4 cy
   bset  FLCR_PGM,MAP_ADDR_6(FLCR)  ; 5 cy     ; Enable Flash programming
   lda   DestinationPage            ; 3 cy     ; Set window to page to program
   sta   PAGESEL                    ; 3 cy  
   lda   DestinationAddr            ; 3 cy     ; Dummy write to Flash block
   tax                              ; 2 cy
   sta   ,X                         ; 2 cy
                                   ;=======

   lda   Time7us                    ; 3 cy     ; Wait Tnvs 
   dbnza *                          ; 4*N cy      
   mov   #HIGH_6_13(FLCR),PAGESEL   ; 4 cy
   bset  FLCR_HVEN,MAP_ADDR_6(FLCR) ; 5 cy     ; Enable Flash high voltage
                                   ;======= Tnvs = 12+4*N, 28cy =~7us

   lda   Time12us                   ; 3 cy     ; Wait Tpgs
   dbnza *                          ; 4*N cy           

   lda   DestinationPage            ; 3 cy     ; Set window to page to program
   sta   PAGESEL                    ; 3 cy
                                   ;======= Tpgs = 24+4*N, 48cy = ~12us 
                                   ;             Measured to actual write below
Loop:
   lda   SourceAddr                 ; 3 cy     ; Copy byte to flash (program)
   tax                              ; 2 cy
   lda   ,X                         ; 3 cy
   ldx   DestinationAddr            ; 5 cy
   sta   ,X                         ; 2 cy 
                                    ; -----
   lda   Time30us                   ; 3 cy     ; Wait Tprog
   dbnza *                          ; 4*N cy
   
   inc   DestinationAddr            ; 4 cy     ; Update ptrs/counter
   dbnz  SourceAddr,Loop            ; 7 cy     ; Finished? - no, loop
                                   ;======= Tprog = 29+4*N, 81cy = ~30.25us
   
   mov   #HIGH_6_13(FLCR),PAGESEL   ; 4 cy
   bclr  FLCR_PGM,MAP_ADDR_6(FLCR)  ; 5 cy     ; Disable Flash programming
                                   ;======= Tprog(last) = 23+4*N, 75cy = ~28.75us

   lda   Time7us                    ; 3 cy     ; Wait Tnvh
   dbnza *                          ; 4*N cy     

   bclr  FLCR_HVEN,MAP_ADDR_6(FLCR) ; 5 cy     ; Disable High voltage
                                   ;======= Tnvh = 8+4*N, 24cy = ~6us
   
   bgnd                             ; 5+ cy    ; Suspend - BDM looks for this
   
   org   $3FFD                      ;          ; Dummy reset vector for simulation
   jmp   RAMStart